@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------------
 //
 //  Little Color Management System
-//  Copyright (c) 1998-2010 Marti Maria Saguer
+//  Copyright (c) 1998-2011 Marti Maria Saguer
 //
 // Permission is hereby granted, free of charge, to any person obtaining 
 // a copy of this software and associated documentation files (the "Software"), 
@@ -26,8 +26,12 @@
 
 #include "utils.h"
 
+#ifndef _MSC_VER 
+#    include <unistd.h> 
+#endif 
+
 #ifdef CMS_IS_WINDOWS_
-#include <io.h>
+#    include <io.h>
 #endif
 
 #define MAX_INPUT_BUFFER 4096
@@ -40,6 +44,7 @@ static cmsBool           Width16                = FALSE;
 static cmsBool           BlackPointCompensation = FALSE;
 static cmsBool           lIsDeviceLink          = FALSE;
 static cmsBool           lQuantize              = FALSE;
+static cmsBool           lUnbounded             = TRUE;
 static cmsBool           lIsFloat               = TRUE;
 
 static cmsUInt32Number   Intent           = INTENT_PERCEPTUAL;
@@ -95,9 +100,10 @@ void Help(void)
 
     fprintf(stderr, "%ce[op] - Encoded representation of numbers\n", SW);
     fprintf(stderr, "\t%cw - use 16 bits\n", SW);     
-    fprintf(stderr, "\t%cx - Hexadecimal\n", SW);
-    fprintf(stderr, "%cq - Quantize CGATS to 8 bits\n\n", SW);
+    fprintf(stderr, "\t%cx - Hexadecimal\n\n", SW);
 
+    fprintf(stderr, "%cs - bounded mode (clip negatives and highliths)\n", SW);
+    fprintf(stderr, "%cq - Quantize (round decimals)\n\n", SW);
 
     fprintf(stderr, "%ci<profile> - Input profile (defaults to sRGB)\n", SW);
     fprintf(stderr, "%co<profile> - Output profile (defaults to sRGB)\n", SW);   
@@ -139,7 +145,7 @@ void HandleSwitches(int argc, char *argv[])
     int s;
 
     while ((s = xgetopt(argc, argv,
-        "bBC:c:d:D:eEgGI:i:L:l:m:M:nNO:o:p:P:QqT:t:V:v:WwxX!:")) != EOF) {
+        "bBC:c:d:D:eEgGI:i:L:l:m:M:nNO:o:p:P:QqSsT:t:V:v:WwxX!:")) != EOF) {
 
     switch (s){
 
@@ -162,7 +168,7 @@ void HandleSwitches(int argc, char *argv[])
         case 'd':
         case 'D': {
             cmsFloat64Number ObserverAdaptationState = atof(xoptarg);
-            if (ObserverAdaptationState < 0 && 
+            if (ObserverAdaptationState < 0 || 
                 ObserverAdaptationState > 1.0)
                 FatalError("Adaptation states should be between 0 and 1");
 
@@ -222,12 +228,18 @@ void HandleSwitches(int argc, char *argv[])
             cProofing = xoptarg;
             break;      
 
-            // Quantize to 16 bits
+            // Quantize (get rid of decimals)
         case 'q':
         case 'Q': 
             lQuantize = TRUE;
             break;
 
+            // Inhibit unbounded mode
+        case 's':
+        case 'S':
+               lUnbounded = FALSE;
+               break;
+
             // The intent
         case 't':
         case 'T':
@@ -455,13 +467,22 @@ cmsBool OpenTransforms(void)
         if (cmsIsTag(hInput, cmsSigColorantTableTag)) {
             List = cmsReadTag(hInput, cmsSigColorantTableTag);
             InputColorant = cmsDupNamedColorList(List);
+            if (cmsNamedColorCount(InputColorant) <= 3) 
+                SetRange(255, TRUE);
+            else
+                SetRange(1, TRUE);  // Inks are already divided by 100 in the formatter
+
         }
         else InputColorant = ComponentNames(InputColorSpace, TRUE);
 
         if (cmsIsTag(hOutput, cmsSigColorantTableTag)){
 
-            List = cmsReadTag(hInput, cmsSigColorantTableTag);
+            List = cmsReadTag(hOutput, cmsSigColorantTableTag);
             OutputColorant = cmsDupNamedColorList(List);
+            if (cmsNamedColorCount(OutputColorant) <= 3) 
+                SetRange(255, FALSE);
+            else
+                SetRange(1, FALSE);  // Inks are already divided by 100 in the formatter
         }
         else OutputColorant = ComponentNames(OutputColorSpace, FALSE);
 
@@ -640,7 +661,7 @@ void PrintFloatResults(cmsFloat64Number Value[])
         }
         else {
             OutputRange = 1;
-            sprintf(ChannelName, "Channel #%d", i + 1);
+            sprintf(ChannelName, "Channel #%u", i + 1);
         }
 
         v = (cmsFloat64Number) Value[i]* OutputRange;
@@ -648,6 +669,14 @@ void PrintFloatResults(cmsFloat64Number Value[])
         if (lQuantize) 
             v = floor(v + 0.5);
 
+        if (!lUnbounded) {
+
+               if (v < 0)
+                      v = 0;
+               if (v > OutputRange)
+                      v = OutputRange;
+        }
+
         if (Verbose <= 0)
             printf("%.4f ", v);
         else
@@ -711,7 +740,7 @@ void TakeFloatValues(cmsFloat64Number Float[])
         }
         else {
             InputRange = 1;
-            sprintf(ChannelName, "Channel #%d", i+1);
+            sprintf(ChannelName, "Channel #%u", i+1);
         }
 
         GetLine(Buffer, "%s? ", ChannelName);
@@ -760,7 +789,7 @@ void PrintEncodedResults(cmsUInt16Number Encoded[])
             cmsNamedColorInfo(OutputColorant, i, ChannelName, NULL, NULL, NULL, NULL);          
         }
         else {          
-            sprintf(ChannelName, "Channel #%d", i + 1);
+            sprintf(ChannelName, "Channel #%u", i + 1);
         }
 
         if (Verbose > 0)
@@ -829,7 +858,7 @@ cmsFloat64Number GetIT8Val(const char* Name, cmsFloat64Number Max)
 // Read input values from CGATS file.
 
 static
-void TakeCGATSValues(int nPatch, cmsFloat64Number Float[])
+    void TakeCGATSValues(int nPatch, cmsFloat64Number Float[])
 {
 
     // At first take the name if SAMPLE_ID is present
@@ -843,15 +872,15 @@ void TakeCGATSValues(int nPatch, cmsFloat64Number Float[])
 
     if (InputNamedColor) {
 
-      const cmsNAMEDCOLORLIST* NamedColorList;
-      int index;
+        const cmsNAMEDCOLORLIST* NamedColorList;
+        int index;
 
-      NamedColorList = cmsGetNamedColorList(hTrans);
-      if (NamedColorList == NULL) 
-          FatalError("Malformed named color profile");
-      
-      index = cmsNamedColorIndex(NamedColorList, CGATSPatch);
-      if (index < 0) 
+        NamedColorList = cmsGetNamedColorList(hTrans);
+        if (NamedColorList == NULL) 
+            FatalError("Malformed named color profile");
+
+        index = cmsNamedColorIndex(NamedColorList, CGATSPatch);
+        if (index < 0) 
             FatalError("Named color '%s' not found in the profile", CGATSPatch); 
 
         Float[0] = index;
@@ -900,6 +929,36 @@ void TakeCGATSValues(int nPatch, cmsFloat64Number Float[])
         Float[2] = GetIT8Val("CMY_Y", 1.0);
         break;
 
+    case cmsSig1colorData:
+    case cmsSig2colorData:
+    case cmsSig3colorData:
+    case cmsSig4colorData:
+    case cmsSig5colorData:
+    case cmsSig6colorData:
+    case cmsSig7colorData:
+    case cmsSig8colorData:
+    case cmsSig9colorData:
+    case cmsSig10colorData:
+    case cmsSig11colorData:
+    case cmsSig12colorData:
+    case cmsSig13colorData:
+    case cmsSig14colorData:
+    case cmsSig15colorData:
+        {
+            cmsUInt32Number i, n;
+
+            n = cmsChannelsOf(InputColorSpace);
+            for (i=0; i < n; i++) { 
+
+                char Buffer[255];
+
+                sprintf(Buffer, "%uCLR_%u", n, i+1);
+                Float[i] = GetIT8Val(Buffer, 100.0);
+            }
+
+        }
+        break;
+
     default: 
         {
             cmsUInt32Number i, n;
@@ -909,7 +968,7 @@ void TakeCGATSValues(int nPatch, cmsFloat64Number Float[])
 
                 char Buffer[255];
 
-                sprintf(Buffer, "CHAN_%d", i+1);
+                sprintf(Buffer, "CHAN_%u", i+1);
                 Float[i] = GetIT8Val(Buffer, 1.0);
             }
 
@@ -978,6 +1037,37 @@ void PutCGATSValues(cmsFloat64Number Float[])
         SetCGATSfld("CMY_Y", Float[2]);                 
         break;
 
+    case cmsSig1colorData:
+    case cmsSig2colorData:
+    case cmsSig3colorData:
+    case cmsSig4colorData:
+    case cmsSig5colorData:
+    case cmsSig6colorData:
+    case cmsSig7colorData:
+    case cmsSig8colorData:
+    case cmsSig9colorData:
+    case cmsSig10colorData:
+    case cmsSig11colorData:
+    case cmsSig12colorData:
+    case cmsSig13colorData:
+    case cmsSig14colorData:
+    case cmsSig15colorData:
+        {
+
+            cmsUInt32Number i, n;
+
+            n = cmsChannelsOf(InputColorSpace);
+            for (i=0; i < n; i++) { 
+
+                char Buffer[255];
+
+                sprintf(Buffer, "%uCLR_%u", n, i+1);
+
+                SetCGATSfld(Buffer, Float[i] * 100.0);
+            }
+        }
+        break;
+
     default: 
         {
 
@@ -988,7 +1078,7 @@ void PutCGATSValues(cmsFloat64Number Float[])
 
                 char Buffer[255];
 
-                sprintf(Buffer, "CHAN_%d", i+1);
+                sprintf(Buffer, "CHAN_%u", i+1);
 
                 SetCGATSfld(Buffer, Float[i]);
             }
@@ -1065,6 +1155,36 @@ void SetOutputDataFormat(void)
         cmsIT8SetDataFormat(hIT8out, 3, "CMY_Y");                   
         break;
 
+    case cmsSig1colorData:
+    case cmsSig2colorData:
+    case cmsSig3colorData:
+    case cmsSig4colorData:
+    case cmsSig5colorData:
+    case cmsSig6colorData:
+    case cmsSig7colorData:
+    case cmsSig8colorData:
+    case cmsSig9colorData:
+    case cmsSig10colorData:
+    case cmsSig11colorData:
+    case cmsSig12colorData:
+    case cmsSig13colorData:
+    case cmsSig14colorData:
+    case cmsSig15colorData:
+        {
+            int i, n;
+            char Buffer[255];
+
+            n = cmsChannelsOf(OutputColorSpace);
+            cmsIT8SetPropertyDbl(hIT8out, "NUMBER_OF_FIELDS", n+1);
+            cmsIT8SetDataFormat(hIT8out, 0, "SAMPLE_ID");
+
+            for (i=1; i <= n; i++) {
+                sprintf(Buffer, "%dCLR_%d", n, i);
+                cmsIT8SetDataFormat(hIT8out, i, Buffer);
+            }
+        }
+        break;
+
     default: {
 
         int i, n;
@@ -1120,7 +1240,7 @@ int main(int argc, char *argv[])
 
     int nPatch = 0;
 
-    fprintf(stderr, "LittleCMS ColorSpace conversion calculator - 4.0 [LittleCMS %2.2f]\n", LCMS_VERSION / 1000.0);
+    fprintf(stderr, "LittleCMS ColorSpace conversion calculator - 4.3 [LittleCMS %2.2f]\n", LCMS_VERSION / 1000.0);
 
     InitUtils("transicc");
 
